@@ -95,9 +95,9 @@ class AnalysisForm(forms.Form):
     prev_sem_pred = forms.BooleanField(initial=True,required=False,label='Previous semantic predictability')
     foll_sem_pred = forms.BooleanField(initial=True,required=False,label='Following semantic predictability')
     pause_dist = forms.BooleanField(initial=True,required=False,label='Distance to pauses')
-    sem_pred_window = forms.CharField(required=False,label='Semantic predictability window')
+    sem_pred_window = forms.CharField(blank=True,label='Semantic predictability window')
     sem_pred_window.widget = forms.CheckboxSelectMultiple(choices=SEM_PRED_WINDOW)
-    sem_pred_style = forms.CharField(required=False,label='Semantic predictability style')
+    sem_pred_style = forms.CharField(blank=True,label='Semantic predictability style')
     sem_pred_style.widget = forms.CheckboxSelectMultiple(choices=SEM_PRED_STYLE)
 
     #force_fetch = forms.CharField(label='Force fetch')
