@@ -23,7 +23,12 @@ if 'phonostats' in settings.INSTALLED_APPS:
     from phonostats.utils import getNeighCount,getPhonotacticProb,guessStress
 
 if 'celex' in settings.INSTALLED_APPS:
-    from celex.utils import lookupFreq,lookupCat,categorize_words
+    from celex.utils import categorize_words,get_lexical_info
+
+if 'mysql' in settings.DATABASES['default']['ENGINE']:
+    from .helper import mysql_ur_string_lookup as UR_LOOKUP
+else:
+    from .helper import pg_ur_string_lookup as UR_LOOKUP
 
 from .helper import fetch_buckeye_resource,reorganize
 from .managers import BulkManager
@@ -434,17 +439,29 @@ class WordType(caching.base.CachingMixin,models.Model):
         self.save()
         return self.Count
 
-    def getFreq(self):
-        if self.Frequency is not None:
+    def getFreq(self,subset='all',speaker=None,dialog=None):
+        if subset=='all' and self.Frequency is not None:
             return self.Frequency
-        self.Frequency = float(WordToken.objects.filter(WordType=self).count())/float(WordToken.objects.filter(~Q(Category__CategoryType='Other'),~Q(Category__CategoryType='Pause'),~Q(Category__CategoryType='Disfluency')).count())
-        self.Frequency = math.log(self.Frequency * 1000000)
-        self.FreqSource = 'Buckeye'
-        self.save()
-        return self.Frequency
+        if subset == 'gender' and speaker is not None:
+            base = WordToken.objects.filter(Dialog__Speaker__Gender = speaker.Gender)
+        elif subset == 'age' and speaker is not None:
+            base = WordToken.objects.filter(Dialog__Speaker__Age = speaker.Age)
+        elif subset == 'speaker' and speaker is not None:
+            base = WordToken.objects.filter(Dialog__Speaker = speaker)
+        elif subset == 'dialog' and dialog is not None:
+            base = WordToken.objects.filter(Dialog = dialog)
+        else:
+            base = WordToken.objects.all()
 
-    def getNDs(self):
-        if self.ND is not None and self.FWND is not None:
+        Freq = float(base.filter(WordType=self).count())/float(base.filter(~Q(Category__CategoryType='Other'),~Q(Category__CategoryType='Pause'),~Q(Category__CategoryType='Disfluency')).count())
+        Freq = math.log(Freq * 1000000,10)
+        if subset=='all':
+            self.Frequency = Freq
+            self.save()
+        return Freq
+
+    def getNDs(self,subset='all',speaker=None,dialog=None):
+        if subset == 'all' and self.ND is not None and self.FWND is not None:
             return self.ND,self.FWND
         any_segment = '[A-Za-z]{1,2}'
         phones = map(str,self.UR.all())
@@ -462,40 +479,30 @@ class WordType(caching.base.CachingMixin,models.Model):
             patt.extend(phones[i:])
             patterns.append('^'+' '.join(patt) +'$')
         neighs = WordType.objects.filter(Label__regex="^[^{<]").extra(
-                    where = ["""(SELECT GROUP_CONCAT(buckeyebrowser_segmenttype.Label SEPARATOR ' ')
-                                FROM buckeyebrowser_underlying
-                                INNER JOIN buckeyebrowser_segmenttype
-                                ON buckeyebrowser_segmenttype.id
-                                    = buckeyebrowser_underlying.SegmentType_id
-                                WHERE buckeyebrowser_underlying.WordType_id
-                                    =buckeyebrowser_wordtype.id) REGEXP %s"""],
+                    where = [UR_LOOKUP],
                     params = ['|'.join(patterns)])
         nd = len(neighs)
-        fwnd = sum([ x.getFreq() for x in neighs])
-        self.ND = nd
-        self.FWND = fwnd
-        self.save()
-        return self.ND,self.FWND
+        fwnd = sum([ x.getFreq(subset=subset,speaker=speaker,dialog=dialog) for x in neighs])
+        if subset == 'all':
+            self.ND,self.FWND = nd,fwnd
+            self.save()
+        return nd,fwnd
 
-    def getND(self):
-        if self.ND is not None:
+    def getND(self,subset='all',speaker=None,dialog=None):
+        if subset == 'all' and self.ND is not None:
             return self.ND
-        nd,fwnd = self.getNDs()
+        nd,fwnd = self.getNDs(subset=subset,speaker=speaker,dialog=dialog)
         return nd
 
-    def getFWND(self):
-        if self.FWND is not None:
+    def getFWND(self,subset='all',speaker=None,dialog=None):
+        if subset == 'all' and self.FWND is not None:
             return self.FWND
-        nd,fwnd = self.getNDs()
+        nd,fwnd = self.getNDs(subset=subset,speaker=speaker,dialog=dialog)
         return fwnd
 
-    def getPhonoProb(self):
-        if self.SPhonoProb is None:
-            self.setViteLuceProb()
-
-        return self.SPhonoProb,self.BiPhonoProb
-
-    def setViteLuceProb(self):
+    def getPhonoProb(self,subset='all',speaker=None,dialog=None):
+        if self.SPhonoProb is not None and self.BiPhonoProb is not None:
+            return self.SPhonoProb,self.BiPhonoProb
         any_segment = '[A-Za-z]{1,2}'
         patterns = []
         SPprob = 0.0
@@ -507,52 +514,33 @@ class WordType(caching.base.CachingMixin,models.Model):
             pattern = '^'+' '.join(patt) +'.*$'
             totPattern = '^'+' '.join([any_segment] * (i+1)) +'.*$'
             count = WordType.objects.filter(Label__regex="^[^{<]").extra(
-                    where = ["""(SELECT GROUP_CONCAT(buckeyebrowser_segmenttype.Label SEPARATOR ' ')
-                                FROM buckeyebrowser_underlying
-                                INNER JOIN buckeyebrowser_segmenttype
-                                ON buckeyebrowser_segmenttype.id
-                                    = buckeyebrowser_underlying.SegmentType_id
-                                WHERE buckeyebrowser_underlying.WordType_id
-                                    =buckeyebrowser_wordtype.id) REGEXP %s"""],
-                    params = [pattern]).count()
+                    where = [UR_LOOKUP],
+                    params = [pattern])
             totCount = WordType.objects.filter(Label__regex="^[^{<]").extra(
-                    where = ["""(SELECT GROUP_CONCAT(buckeyebrowser_segmenttype.Label SEPARATOR ' ')
-                                FROM buckeyebrowser_underlying
-                                INNER JOIN buckeyebrowser_segmenttype
-                                ON buckeyebrowser_segmenttype.id
-                                    = buckeyebrowser_underlying.SegmentType_id
-                                WHERE buckeyebrowser_underlying.WordType_id
-                                    =buckeyebrowser_wordtype.id) REGEXP %s"""],
-                    params = [totPattern]).count()
-            SPprob += float(count) / float(totCount)
+                    where = [UR_LOOKUP],
+                    params = [totPattern])
+            SPprob += float(sum([x.getFreq(subset=subset,speaker=speaker,dialog=dialog)
+                                for x in count])) / float(sum([x.getFreq(subset=subset,speaker=speaker,dialog=dialog)
+                                                                    for x in totCount]))
             if i != len(phones)-1:
                 patt = [any_segment] * i
                 patt.extend([phones[i],phones[i+1]])
                 pattern = '^'+' '.join(patt) +'.*$'
                 totPattern = '^'+' '.join([any_segment] * (i+2)) +'.*$'
                 count = WordType.objects.filter(Label__regex="^[^{<]").extra(
-                    where = ["""(SELECT GROUP_CONCAT(buckeyebrowser_segmenttype.Label SEPARATOR ' ')
-                                FROM buckeyebrowser_underlying
-                                INNER JOIN buckeyebrowser_segmenttype
-                                ON buckeyebrowser_segmenttype.id
-                                    = buckeyebrowser_underlying.SegmentType_id
-                                WHERE buckeyebrowser_underlying.WordType_id
-                                    =buckeyebrowser_wordtype.id) REGEXP %s"""],
+                    where = [UR_LOOKUP],
                     params = [pattern]).count()
                 totCount = WordType.objects.filter(Label__regex="^[^{<]").extra(
-                    where = ["""(SELECT GROUP_CONCAT(buckeyebrowser_segmenttype.Label SEPARATOR ' ')
-                                FROM buckeyebrowser_underlying
-                                INNER JOIN buckeyebrowser_segmenttype
-                                ON buckeyebrowser_segmenttype.id
-                                    = buckeyebrowser_underlying.SegmentType_id
-                                WHERE buckeyebrowser_underlying.WordType_id
-                                    =buckeyebrowser_wordtype.id) REGEXP %s"""],
+                    where = [UR_LOOKUP],
                     params = [totPattern]).count()
-                BPprob += float(count) / float(totCount)
+                BPprob += float(sum([x.getFreq(subset=subset,speaker=speaker,dialog=dialog)
+                                for x in count])) / float(sum([x.getFreq(subset=subset,speaker=speaker,dialog=dialog)
+                                                                    for x in totCount]))
         SPprob = SPprob / float(len(phones))
         BPprob = BPprob / float(len(phones)-1)
         self.SPhonoProb,self.BiPhonoProb = SPprob,BPprob
         self.save()
+        return SPprob,BPprob
 
 
 class WordToken(caching.base.CachingMixin,models.Model):
@@ -614,7 +602,7 @@ class WordToken(caching.base.CachingMixin,models.Model):
                                     if str(x) != str(self)]
                                 + [str(self)])
         cat_word = catted.pop(-1)
-        tagged_context = [x for x in
+        tagged_context = ['#'.join([x,'1']) for x in
                             catted
                             if x[-2] == '#' and x[-1] in ['n','v','a','r']]
         if cat_word.endswith('NA'):
@@ -629,7 +617,7 @@ class WordToken(caching.base.CachingMixin,models.Model):
                 cat = 'r'
             else:
                 cat = 'NA'
-            cat_word  = '#'.join([cat_word.split("#")[0],cat])
+            cat_word  = '#'.join([cat_word.split("#")[0],cat,'1'])
         sp = getSemanticRelatedness(cat_word,tagged_context,style=style)
         if style == 'A' and window == 'A':
             self.PrevSemPred = sp
@@ -650,7 +638,7 @@ class WordToken(caching.base.CachingMixin,models.Model):
                                     if str(x) != str(self)]
                                 + [str(self)])
         cat_word = catted.pop(-1)
-        tagged_context = [x for x in
+        tagged_context = ['#'.join([x,'1']) for x in
                             catted
                             if x[-2] == '#' and x[-1] in ['n','v','a','r']]
         if cat_word.endswith('NA'):
@@ -665,7 +653,7 @@ class WordToken(caching.base.CachingMixin,models.Model):
                 cat = 'r'
             else:
                 cat = 'NA'
-            cat_word  = '#'.join([cat_word.split("#")[0],cat])
+            cat_word  = '#'.join([cat_word.split("#")[0],cat,'1'])
         sp = getSemanticRelatedness(cat_word,tagged_context,style=style)
         if style == 'A' and window == 'A':
             self.FollSemPred = sp
@@ -996,7 +984,6 @@ class WordToken(caching.base.CachingMixin,models.Model):
         form = output_form.cleaned_data
         wanted_fields = output_form.get_wanted_fields()
         if self.Output is not None:
-            #outline = {x: self.Output[x] for x in self.Output if x not in form['force_fetch']}
             outline = copy.copy(self.Output)
         else:
             outline = {}
@@ -1035,25 +1022,40 @@ class WordToken(caching.base.CachingMixin,models.Model):
                 else:
                     outline['wasRepeated'] = 'False'
 
+        if 'C' in form['category'] + form['lexical_scale']:
+            celex_info = get_lexical_info(self.WordType.Label)
 
         if 'B' in form['category']:
             if 'BuckeyeCategory' not in outline:
                 outline['BuckeyeCategory'] = self.Category.Label
         if 'C' in form['category']:
             if 'CelexCategory' not in outline:
-                outline['CelexCategory'] = lookupCat(self.WordType.Label)
+                try:
+                    outline['CelexCategory'] = celex_info['Cat']
+                except KeyError:
+                    outline['CelexCategory'] = 'NA'
 
         if form['wordDuration']:
             if 'WordDuration' not in outline:
                 outline['WordDuration'] = self.getDuration()
                 outline['BaselineDur'] = self.WordType.getBaseDuration()
 
-        if 'B' in form['frequency']:
-            if 'BuckeyeFrequency' not in outline:
+        if form['frequency']:
+            if 'C' in form['lexical_scale'] and 'CelexFrequency' not in outline:
+                try:
+                    outline['CelexFrequency'] = celex_info['Freq']
+                except KeyError:
+                    outline['CelexFrequency'] = 'NA'
+            if 'W' in form['lexical_scale'] and 'BuckeyeFrequency' not in outline:
                 outline['BuckeyeFrequency'] = self.WordType.getFreq()
-        if 'C' in form['frequency']:
-            if 'CelexFrequency' not in outline:
-                outline['CelexFrequency'] = lookupFreq(self.WordType.Label)
+            if 'S' in form['lexical_scale'] and 'SpeakerFrequency' not in outline:
+                outline['SpeakerFrequency'] = self.WordType.getFreq(subset='speaker',speaker=self.Dialog.Speaker)
+            if 'A' in form['lexical_scale'] and 'AgeFrequency' not in outline:
+                outline['AgeFrequency'] = self.WordType.getFreq(subset='age',speaker=self.Dialog.Speaker)
+            if 'G' in form['lexical_scale'] and 'GenderFrequency' not in outline:
+                outline['GenderFrequency'] = self.WordType.getFreq(subset='gender',speaker=self.Dialog.Speaker)
+            if 'D' in form['DialogFrequency'] and 'DialogFrequency' not in outline:
+                outline['DialogNeighDen'] = self.WordType.getFreq(subset='dialog',speaker=self.Dialog.Speaker,dialog=self.Dialog)
 
         if form['gender']:
             if 'SpeakerGender' not in outline:
@@ -1068,22 +1070,51 @@ class WordToken(caching.base.CachingMixin,models.Model):
         if form['phonoLength']:
             if 'PhonoLength' not in outline:
                 outline['PhonoLength'] = self.WordType.UR.count()
-        if 'B' in form['nd']:
-            if 'BuckeyeNeighDen' not in outline:
+        if form['nd']:
+            if 'C' in form['lexical_scale'] and 'CelexNeighDen' not in outline:
+                try:
+                    outline['CelexNeighDen','CelexFWND'] = celex_info['ND'],celex_info['FWND']
+                except KeyError:
+                    outline['CelexNeighDen','CelexFWND'] = 'NA','NA'
+            if 'W' in form['lexical_scale'] and 'BuckeyeNeighDen' not in outline:
                 outline['BuckeyeNeighDen'],outline['BuckeyeFWND'] = self.WordType.getNDs()
-        if 'PS' in form['nd']:
-            if 'PhonoStatsNeighDen' not in outline:
-                outline['PhonoStatsNeighDen'] = getNeighCount(self.WordType.getStrUR(),no_stress=True)
+            if 'S' in form['lexical_scale'] and 'SpeakerNeighDen' not in outline:
+                outline['SpeakerNeighDen'],outline['SpeakerFWND'] = self.WordType.getNDs(subset='speaker',speaker=self.Dialog.Speaker)
+            if 'A' in form['lexical_scale'] and 'AgeNeighDen' not in outline:
+                outline['AgeNeighDen'],outline['AgeFWND'] = self.WordType.getNDs(subset='age',speaker=self.Dialog.Speaker)
+            if 'G' in form['lexical_scale'] and 'GenderNeighDen' not in outline:
+                outline['GenderNeighDen'],outline['GenderFWND'] = self.WordType.getNDs(subset='gender',speaker=self.Dialog.Speaker)
+            if 'D' in form['lexical_scale'] and 'DialogNeighDen' not in outline:
+                outline['DialogNeighDen'],outline['DialogFWND'] = self.WordType.getNDs(subset='dialog',speaker=self.Dialog.Speaker,dialog=self.Dialog)
 
-        if 'VB' in form['phonotactic']:
-            if 'BuckeyeSPhoneProb' not in outline:
+        if form['phonotactic']:
+            if 'C' in form['lexical_scale'] and 'CelexSPhoneProb' not in outline:
+                try:
+                    outline['CelexSPhoneProb','CelexBiPhoneProb'] = celex_info['SP'],celex_info['BP']
+                except KeyError:
+                    outline['CelexSPhoneProb','CelexBiPhoneProb'] = 'NA','NA'
+            if 'W' in form['lexical_scale'] and 'BuckeyeSPhoneProb' not in outline:
                 outline['BuckeyeSPhoneProb'],outline['BuckeyeBiPhoneProb'] = self.WordType.getPhonoProb()
-        if 'B' in form['phonotactic']:
+            if 'S' in form['lexical_scale'] and 'SpeakerSPhoneProb' not in outline:
+                outline['SpeakerSPhoneProb'],outline['SpeakerBiPhoneProb'] = self.WordType.getPhonoProb(subset='speaker',speaker=self.Dialog.Speaker)
+            if 'A' in form['lexical_scale'] and 'AgeSPhoneProb' not in outline:
+                outline['AgeSPhoneProb'],outline['AgeBiPhoneProb'] = self.WordType.getPhonoProb(subset='age',speaker=self.Dialog.Speaker)
+            if 'G' in form['lexical_scale'] and 'GenderSPhoneProb' not in outline:
+                outline['GenderSPhoneProb'],outline['GenderBiPhoneProb'] = self.WordType.getPhonoProb(subset='gender',speaker=self.Dialog.Speaker)
+            if 'D' in form['lexical_scale'] and 'DialogSPhoneProb' not in outline:
+                outline['DialogSPhoneProb'],outline['DialogBiPhoneProb'] = self.WordType.getPhonoProb(subset='dialog',speaker=self.Dialog.Speaker,dialog=self.Dialog)
+
+        if form['additional_phono_stats']:
+            if 'PhonoStatsNeighDen' not in outline:
+                outline['PhonoStatsNeighDen'] = getNeighCount(self.WordType.getUR(
+                                                        stressed=True,blick_style=True),no_stress=True)
             if 'PhonoStatsBlickPhono' not in outline:
-                outline['PhonoStatsBlickPhono'] = getPhonotacticProb(self.WordType.getStrUR(),use_blick=True,no_stress=True)
-        if 'VP' in form['phonotactic']:
+                outline['PhonoStatsBlickPhono'] = getPhonotacticProb(self.WordType.getUR(
+                                                        stressed=True,blick_style=True),
+                                                        use_blick=True,no_stress=True)
             if 'PhonoStatsSPhoneProb' not in outline:
-                outline['PhonoStatsSPhoneProb'],outline['PhonoStatsBiPhoneProb'] = getPhonotacticProb(self.WordType.getStrUR(),use_blick=False,no_stress=True)
+                outline['PhonoStatsSPhoneProb'],outline['PhonoStatsBiPhoneProb'] = getPhonotacticProb(self.WordType.getUR(
+                                                        stressed=True,blick_style=True),use_blick=False,no_stress=True)
 
         for w in eval(form['sem_pred_window']):
             for s in eval(form['sem_pred_style']):
